#======================================================================
# Model planowania produkcji dla zadanego planu dystrybucji (transportu) w ramach VMI
#======================================================================

param Model symbolic := "VMI_D_P";

param St_wykorzystania >= 0, <= 1;
param Dane symbolic :=
	"VMI_D_P_0"&substr(St_wykorzystania,3)&".csv";
#======================================================================
param L_dni > 0;
set Dni := 1..L_dni;

set Produkty;
set Samochody;
set Platformy;

#======================================================================

param Jedn_koszt_zapasow >= 0;
param Koszt_przezbr >= 0;
param Koszt_str_popytu >= 0;

param Czas_wyk {Produkty} >= 0;
param Pocz_gotowosc {Produkty} binary;

param Pocz_zapas_dostawcy {Produkty} >= 0;
param Zapas_bezp {Platformy, Produkty} >= 0;
param Pocz_zapas_odbiorcy {Platformy, Produkty} >= 0;
param Popyt {Platformy, Produkty, t in Dni} >= 0;

table T_Pocz_zapas_dostawcy	IN "CSV" "Pocz_zapas_dostawcy.csv":
	[Produkt], Pocz_zapas_dostawcy~Pocz_zapas_dostawcy;

table T_Zapas_bezp IN "CSV" "Zapas_bezp.csv":
	[Platforma, Produkt], Zapas_bezp~Zapas_bezp	;

table T_Pocz_zapas_odbiorcy IN "CSV" "Pocz_zapas_odbiorcy.csv":
	[Platforma, Produkt], Pocz_zapas_odbiorcy~Pocz_zapas_odbiorcy	;

table T_Popyt IN "CSV" "Popyt.csv":
	[Platforma, Produkt, Dzien], Popyt~Popyt;

param Koszt_transp {Samochody} >= 0;
param Ladownosc {Samochody} >= 0;
param Czas_transp {Samochody} >= 0;
param min_Czas_transp := min {s in Samochody} Czas_transp[s];
param max_Czas_transp := max {s in Samochody} Czas_transp[s];

param Obciazenie :=
	sum {j in Produkty} Czas_wyk[j] *
		(sum {p in Platformy} (sum {t in Dni} Popyt[p, j, t]	 - Pocz_zapas_odbiorcy[p, j])
		- Pocz_zapas_dostawcy[j]);

param Czas_pracy := Obciazenie / (L_dni-min_Czas_transp) / St_wykorzystania;

#======================================================================
var Produkcja {Produkty, Dni} >= 0;
var Uruchomienie {Produkty, Dni} binary;
var Gotowosc {Produkty, Dni union {0}} binary;
var Zapas_dostawcy {j in Produkty, Dni union {0}} >= 0;

param Transport {Platformy, Samochody, Dni} binary;
var Wielkosc_transp {Platformy, Samochody, Dni, Produkty} >= 0;
var Zapas_odbiorcy {p in Platformy, j in Produkty, Dni union {0}} >= Zapas_bezp[p,j];
var Stracony_popyt {p in Platformy, j in Produkty, Dni union {0}} >= 0;

var Koszty_przezbrajania;
var Koszty_zapasow_dostawcy;
var Koszty_transportu;
var Koszty_zapasow_w_drodze;
var Koszty_zapasow_odbiorcy;
var Koszty_straconego_popytu;
var Koszty_calk;

table transport IN "CSV" "transport.csv":
	[Platforma, Samochod, Dzien], Transport~Transport;

#======================================================================

minimize FC_Koszty_calk:
					Koszty_calk;

#======================================================================
subject to


	Ogr_Koszty_calk:
		Koszty_calk =
				Koszty_przezbrajania
			+	Koszty_zapasow_dostawcy
			+	Koszty_transportu
			+	Koszty_zapasow_w_drodze
			+	Koszty_zapasow_odbiorcy
			+	Koszty_straconego_popytu;

	Ogr_Koszty_przezbrajania:
		Koszty_przezbrajania =
			sum {j in Produkty, t in Dni}
				Koszt_przezbr * Uruchomienie[j, t];

	Ogr_Koszty_zapasow_dostawcy:
		Koszty_zapasow_dostawcy =
			sum {j in Produkty, t in Dni}
				Jedn_koszt_zapasow * Zapas_dostawcy[j, t];

	Ogr_Koszty_transportu:
		Koszty_transportu =
			sum {p in Platformy, s in Samochody, t in Dni}
				Koszt_transp[s] * Transport[p, s, t];

	Ogr_Koszty_zapasow_w_drodze:
		Koszty_zapasow_w_drodze =
			sum {p in Platformy, s in Samochody, t in Dni, j in Produkty}
				Jedn_koszt_zapasow * Czas_transp[s] * Wielkosc_transp[p, s, t, j];

	Ogr_Koszty_zapasow_odbiorcy:
		Koszty_zapasow_odbiorcy =
			sum {p in Platformy, j in Produkty, t in Dni}
				Jedn_koszt_zapasow * Zapas_odbiorcy[p, j, t];

	Ogr_Koszty_straconego_popytu:
		Koszty_straconego_popytu =
			sum {p in Platformy, j in Produkty, t in Dni}
				Koszt_str_popytu * Stracony_popyt[p, j, t]	;

	#------------------------------------------------------------------------------------------------------------------------
	# Produkcja
	#------------------------------------------------------------------------------------------------------------------------

	Ogr_Pocz_zapas_dostawcy {j in Produkty}:
		Zapas_dostawcy[j, 0] = Pocz_zapas_dostawcy[j];

	Ogr_Zdolnosc_prod {t in Dni}:
		sum {j in Produkty} Czas_wyk[j] * Produkcja[j, t] <= Czas_pracy;

	Ogr_Gotowosc {j in Produkty, t in Dni}:
		Czas_wyk[j] * Produkcja[j, t] <= Czas_pracy * (Gotowosc[j, t-1] + Gotowosc[j, t]);

	Ogr_Jednoznaczna_gotowosc {t in Dni}:
		sum {j in Produkty} Gotowosc[j, t] <= 1;

	Ogr_Stan_pocz {j in Produkty}:
		Gotowosc[j, 0] = Pocz_gotowosc[j];

	Ogr_Uruchomienie {j in Produkty, t in Dni}:
		Gotowosc[j, t] - Gotowosc[j, t-1] <= Uruchomienie[j, t];

	Ogr_Bilans_zapasu_dostwacy {j in Produkty, t in Dni:
				t+max_Czas_transp <= L_dni }:
		Zapas_dostawcy[j, t-1] + Produkcja[j, t] = Zapas_dostawcy[j, t ]
		+ sum {p in Platformy, s in Samochody} Wielkosc_transp[p, s, t, j];

	NO_Uruchomienie1 {j in Produkty, t in Dni}:
		(1 - Gotowosc[j, t-1]) >= Uruchomienie[j, t];

	NO_Uruchomienie2 {j in Produkty, t in Dni}:
		Gotowosc[j, t] >= Uruchomienie[j, t];

	#~ NO_Zapas_dostwacy {j in Produkty, t in Dni, p in {0} union Dni:
						#~ t + min_Czas_transp + p <= L_dni and p <= 5}:
		#~ Zapas_dostawcy[j, t-1] >= sum {s in t..t+p}
				#~ sum {q in Platformy} Popyt[q, j, s+min_Czas_transp ]
					#~ * (1 - Gotowosc[j, t-1] - sum {r in t..s} Uruchomienie[j, r]);

	#------------------------------------------------------------------------------------------------------------------------
	# Dystrybucja
	#------------------------------------------------------------------------------------------------------------------------

	Ogr_Pocz_zapas_odbiorcy {p in Platformy, j in Produkty}:
		Zapas_odbiorcy[p, j, 0] = Pocz_zapas_odbiorcy[p, j];

	Ogr_Bilans_zapasu_odbiorcy {p in Platformy, j in Produkty, t in Dni}:
		Zapas_odbiorcy[p, j, t-1]
			+ sum {s in Samochody: t-Czas_transp[s] >= 1}
					Wielkosc_transp[p, s, t-Czas_transp[s], j]
		= Zapas_odbiorcy[p, j, t] + Popyt[p, j, t] -	Stracony_popyt[p, j, t];

	Ogr_Transport_oraz_ladownosc {p in Platformy, s in Samochody, t in Dni}:
		sum {j in Produkty} Wielkosc_transp[p, s, t, j]
		<= Ladownosc[s] * Transport[p, s, t];

solve;

	#------------------------------------------------------------------------------------------------------------------------
	# Instrukcja dla kolejnego zadania
	#------------------------------------------------------------------------------------------------------------------------

	#~ table przezbrojenia{j in Produkty, t in Dni} OUT "CSV" "Uruchomienie.csv":
		#~ j~Produkt, t~Dzien, Uruchomienie[j,t]~Uruchomienie;

	#~ table gotowosc{j in Produkty, t in Dni union {0} } OUT "CSV" "Gotowosc.csv":
		#~ j~Produkt, t~Dzien, Gotowosc[j,t]~Gotowosc;

	#------------------------------------------------------------------------------------------------------------------------
	# Rozwizanie
	#------------------------------------------------------------------------------------------------------------------------

	#~ printf "Platforma;Samochod;Dzien;Produkt;Zmienna;Wartosc" > Dane;

	printf "\n;;;;%s;%s", 'Model', Model >> Dane;
	printf "\n;;;;%s;%s", 'Data', time2str(gmtime(),'%F %H:%M') >> Dane;

	printf "\n;;;;%s;%3.1f", 'Koszty_calk', Koszty_calk >> Dane;
	printf "\n;;;;%s;%3.1f", 'Koszty_przezbrajania', Koszty_przezbrajania >> Dane;
	printf "\n;;;;%s;%3.1f", 'Koszty_zapasow_dostawcy', Koszty_zapasow_dostawcy >> Dane;
	printf "\n;;;;%s;%3.1f", 'Koszty_transportu', Koszty_transportu >> Dane;
	printf "\n;;;;%s;%3.1f", 'Koszty_zapasow_w_drodze', Koszty_zapasow_w_drodze >> Dane;
	printf "\n;;;;%s;%3.1f", 'Koszty_zapasow_odbiorcy', Koszty_zapasow_odbiorcy >> Dane;
	printf "\n;;;;%s;%3.1f", 'Koszty_straconego_popytu', Koszty_straconego_popytu >> Dane;

	#~ printf "\n;;;;%s;%d", 'L_dni', L_dni >> Dane;
	#~ printf "\n;;;;%s;%3.1f", 'Czas_pracy', Czas_pracy >> Dane;
	#~ printf "\n;;;;%s;%3.1f", 'Jedn_koszt_zapasow ', Jedn_koszt_zapasow >> Dane;
	#~ printf "\n;;;;%s;%3.1f", 'Koszt_przezbr ', Koszt_przezbr >> Dane;
	#~ printf "\n;;;;%s;%3.1f", 'Koszt_str_popytu', Koszt_str_popytu >> Dane;

	#~ for {j in Produkty} {
		#~ printf "\n;;;%s;%s;%3.1f", j, 'Pocz_zapas_dostawcy', Pocz_zapas_dostawcy[j] >> Dane;
		#~ printf "\n;;;%s;%s;%3.1f", j, 'Czas_wyk', Czas_wyk[j] >> Dane;
	#~ }

	#~ for {p in Platformy, j in Produkty} {
		#~ printf "\n%s;;;%s;%s;%3.1f", p, j, 'Pocz_zapas_odbiorcy', Pocz_zapas_odbiorcy[p, j] >> Dane;
		#~ printf "\n%s;;;%s;%s;%3.1f", p, j, 'Zapas_bezp', Zapas_bezp[p, j] >> Dane;
	#~ }

	for {p in Platformy, t in Dni, j in Produkty} {
		#~ printf "\n%s;;%s;%s;%s;%3.1f", p, t, j, 'Popyt', Popyt[p, j, t] >> Dane;
		printf "\n%s;;%s;%s;%s;%3.1f", p, t, j, 'Stracony_popyt', Stracony_popyt[p, j, t] >> Dane;
	}

	#~ for {s in Samochody} {
		#~ printf "\n;%s;;;%s;%3.1f", s, 'Czas_transp', Czas_transp[s] >> Dane;
		#~ printf "\n;%s;;;%s;%3.1f", s, 'Koszt_transp', Koszt_transp[s] >> Dane;
		#~ printf "\n;%s;;;%s;%3.1f", s, 'Ladownosc', Ladownosc[s] >> Dane;
	#~ }

	for {j in Produkty, t in Dni} {
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Produkcja', Produkcja[j, t] >> Dane;
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Uruchomienie', Uruchomienie[j, t] >> Dane;
	}
	for {j in Produkty, t in Dni union {0}}
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Gotowosc', Gotowosc[j, t] >> Dane;

	#~ for {p in Platformy, s in Samochody, t in Dni}
		#~ printf "\n%s;%s;%d;;%s;%3.1f", p, s, t, 'Transport', Transport[p, s, t] >> Dane;

	for {p in Platformy, s in Samochody, t in Dni, j in Produkty}
		printf "\n%s;%s;%d;%s;%s;%3.1f", p, s, t, j, 'Wielkosc_transpD', Wielkosc_transp[p, s, t, j] >> Dane;

	for {p in Platformy, s in Samochody, t in Dni, j in Produkty: t-Czas_transp[s] >= 1}
		printf "\n%s;%s;%d;%s;%s;%3.1f", p, s, t, j, 'Wielkosc_transpO', Wielkosc_transp[p, s, t-Czas_transp[s], j] >> Dane;

	for {j in Produkty, t in Dni union {0}}
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Zapas_dostawcy', Zapas_dostawcy[j, t] >> Dane;

	for {p in Platformy, j in Produkty, t in Dni union {0}}
		printf "\n%s;;%d;%s;%s;%3.1f", p, t, j, 'Zapas_odbiorcy', Zapas_odbiorcy[p, j, t] >> Dane;

#======================================================================
end;
